# -*- coding: utf-8 -*-
# @author: leesoar

"""postcard"""

POP3_ADDR = {
    "tom": "pop.tom.com",
    "gmail": "pop.gmail.com",
    "yahoo": "pop.mail.yahoo.com",
    "outlook": "pop-mail.outlook.com",
    "china": "pop.china.com",
    "21cn": "pop.21cn.com",
    "163": "pop.163.com",
    "126": "pop.126.com",
    "qq": "pop.qq.com",
    "sina": "pop3.sina.com.cn",
    "88": "pop.88.com",
}

SMTP_ADDR = {
    "tom": "smtp.tom.com",
    "gmail": "smtp.gmail.com",
    "yahoo": "smtp.mail.yahoo.com",
    "outlook": "smtp-mail.outlook.com",
    "china": "smtp.china.com",
    "21cn": "smtp.21cn.com",
    "163": "smtp.163.com",
    "126": "smtp.126.com",
    "qq": "smtp.qq.com",
    "sina": "smtp.sina.com.cn",
    "88": "smtp.88.com",
}

IMAP_ADDR = {
    "tom": "imap.tom.com",
    "88": "imap.88.com",
}
